package reversi;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

import revnetwork.ReversiNetwork;

/**
 * <p>
 * Reversi - glowna klasa odpowiedzialna za utworzenie okna rozgrywki i
 * inicjalizacje innych okien.
 * </p>
 * 
 * @author Szymon Lankiewicz
 * @version 1.0
 */
public class Reversi extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static int TOP_SCORE_POSITIONS = 10;

	/**
	 * f - glowne okno gry
	 */
	static JFrame f = new JFrame("Reversi 1.0");

	/**
	 * plansza - panel na ktorym rysowana bedzie plansza i tlo
	 */
	static GamePanel plansza = new GamePanel();

	/**
	 * but1-5 - przyciski menu
	 */
	static JButton but1 = new JButton("Singleplayer");
	static JButton but2 = new JButton("Mulitplayer");
	static JButton but4 = new JButton("Top Scores");
	static JButton but5 = new JButton("Game Over");

	/**
	 * elementy chat'u
	 */
	static JTextArea chatWindow = new JTextArea();

	/**
	 * CONTROL - watek odpowiedzialny za polaczenie z serwerem. W jednym oknie
	 * mozliwe jest tylko jedno polaczenie z serwerem dlatego obiek CONTROL
	 * klasy NetThread jest statyczny;
	 */
	static NetThread CONTROL = null;

	static Configuration okno;
	static TopScore topokno;

	/**
	 * Zmienne dla opcji TOP SCORE
	 */
	static String Login;
	static String[] TOP_SCORE = new String[2*TOP_SCORE_POSITIONS];

	/**
	 * <p>
	 * Metoda main glownej klasy w pliku. Odpowiedzialna za utworzenie glownego
	 * okna gry.
	 * </p>
	 */

	
	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				createAndShowGUI();
			}
		});
	}

	/**
	 * <p>
	 * metoda odpowiedzialna za dodanie i rozmieszczenie wszystkich elementow w
	 * glownym oknie gry.
	 * </p>
	 */
	private static void createAndShowGUI() {

		JComponent newContentPane = new Reversi();
		newContentPane.setOpaque(true); // content panes must be opaque
		f.setContentPane(newContentPane);

		f.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		f.setLayout(null);
		f.setSize(800, 600);

		plansza.setBounds(0, 0, 800, 600);
		plansza.setLayout(null);

		// dodanie sluchaczy do przyciskow menu;
		MenuButtonListener menuL = new MenuButtonListener();
		but1.addActionListener(menuL);
		but2.addActionListener(menuL);
		but4.addActionListener(menuL);
		but5.addActionListener(menuL);

		// rozmieszczenie "na sztywno" elementow glownego okna gry
		but1.setBounds(600, 0, 200, 50);
		but2.setBounds(600, 50, 200, 50);
		but4.setBounds(600, 100, 200, 50);
		but5.setBounds(600, 150, 200, 50);
		chatWindow.setBounds(600, 200, 200, 370);

		// przypisanie obrazkow do przyciskow w glownym oknie gry
		
		but1.setIcon(new ImageIcon(Reversi.class.getResource("/but1.png")));
		but2.setIcon(new ImageIcon(Reversi.class.getResource("/but2.png")));
		but4.setIcon(new ImageIcon(Reversi.class.getResource("/but4.png")));
		but5.setIcon(new ImageIcon(Reversi.class.getResource("/but5.png")));

		// sformatowanie obiektow odpowiedzialnych za czata
		chatWindow.setBackground(Color.BLACK);
		chatWindow.setForeground(Color.WHITE);
		chatWindow.setEditable(false);

		// dodanie elementow do panelu planszy
		plansza.add(but1);
		plansza.add(but2);
		plansza.add(but4);
		plansza.add(but5);
		plansza.add(chatWindow);

		// dodanie planszy do okna
		f.add(plansza);

		f.setVisible(true);
	}

	/**
	 * <p>
	 * metoda wywolywana po wybraniu odpowiedniego przycisku z menu glownego.
	 * Inicjalizuje rozgrywke SinglePlayer;
	 * </p>
	 */
	public static void singlePlayer() {
		okno = new Configuration((byte) 0);
		okno.showWindow();
	}

	/**
	 * <p>
	 * metoda wywolywana po wybraniu odpowiedniego przycisku z menu glownego.
	 * Inicjalizuje rozgrywke MultiPlayer;
	 * </p>
	 */
	public static void multiPlayer() {
		okno = new Configuration((byte) 1);
		okno.showWindow();
	}

	/**
	 * <p>
	 * metoda wywolywana po wybraniu odpowiedniego przycisku z menu glownego.
	 * Wyswietla okno zawierajace loginy 10 najlepszych graczy.
	 * </p>
	 */
	public static void topScore() {
		topokno = new TopScore();
		topokno.showWindow();
	}

	/**
	 * <p>
	 * metoda wywolywana po wybraniu odpowiedniego przycisku z menu glownego.
	 * Zabija watki i zamyka okno gry.
	 * </p>
	 */
	public static void gameOver() {
		if (Reversi.CONTROL != null) {
			if (Reversi.CONTROL.isAlive()) {
				Reversi.CONTROL.killThread();
			}
		}
		f.dispose();
	}
}

/**
 * <p>
 * Listener odpowiedzialny za przechwycenie klikniecia na ktorys z glownych
 * przyciskow w menu i wywolanie odpowiedniej akcji
 * </p>
 * 
 * @author Szymon Lankiewicz
 * @see Reversi#but1
 * @see Reversi#but2
 * @see Reversi#but4
 * @see Reversi#but5
 * @see Reversi#singlePlayer()
 * @see Reversi#multiPlayer()
 * @see Reversi#topScore()
 * @see Reversi#gameOver()
 */
class MenuButtonListener implements ActionListener {

	public void actionPerformed(ActionEvent e) {
		if (e.getSource().equals(Reversi.but1)) {
			Reversi.singlePlayer();
		} else if (e.getSource().equals(Reversi.but2)) {
			Reversi.multiPlayer();
		} else if (e.getSource().equals(Reversi.but4)) {
			Reversi.topScore();
		} else if (e.getSource().equals(Reversi.but5)) {
			Reversi.gameOver();
			Reversi.f.dispose();
		}
	}
}


/**
 * <p>
 * Klasa odpowiedzialna za otworzenie okna umozliwiajacego skonfigurowanie gry i
 * nawiazanie polaczenia z serwerem
 * </p>
 * 
 * @author Szymon Lankiewicz
 * 
 */
class Configuration extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * w - formatka z elementami konfiguracyjnymi
	 */
	static JFrame w = new JFrame("Configuration");

	private JLabel label3 = new JLabel("Server address: ");
	private JLabel label4 = new JLabel("Server port: ");
	private JLabel label5 = new JLabel("User login: ");

	/**
	 * <p>
	 * Adres komputera na ktorym znajduje sie serwer gry
	 * </p>
	 */
	JTextField addressText = new JTextField(10);
	/**
	 * <p>
	 * Port na ktorym dziala serwer
	 * </p>
	 */
	JTextField portText = new JTextField(10);
	JTextField userLogin = new JTextField(10);

	JButton accept = new JButton("Accept");
	JButton cancel = new JButton("Cancel");

	/**
	 * <p>
	 * Konstruktor okna konfiguracji polaczenia/gry
	 * <p>
	 * 
	 * @param type
	 *            - parametr okreslajacy tryb rozgrywki: 0 - SinglePlayer, 1 -
	 *            MultiPlayer
	 */
	public Configuration(byte type) {
		accept.addActionListener(new AcceptListener(type, userLogin,
				addressText, portText));
		cancel.addActionListener(new CancelListener());
		userLogin.addKeyListener(new TextListener(userLogin, 8));
	}

	/**
	 * <p>
	 * Metoda inicjalizujaca okno konfiguracyjne.
	 * </p>
	 */
	public void showWindow() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {

				JComponent newContentPane = new JPanel();
				newContentPane.setOpaque(false);

				w.setContentPane(newContentPane);
				w.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
				w.setLayout(null);

				w.setSize(190, 200);
				showGUI(w);
				w.setVisible(true);
			}
		});
	}

	/**
	 * Metoda rozmieszczajaca elementy w oknie konfiguracyjnym.
	 * 
	 * @param frame
	 *            Okno w ktorym maja byc rozmieszczone elementy.
	 */
	public void showGUI(JFrame frame) {

		addressText.setText("192.168.1.2"); // domyslny adres serwera
		portText.setText("4321"); // domyslny port serwera
		userLogin.setText("GraczPC"); // domyslny login

		frame.setLayout(new FlowLayout());

		frame.add(label3);
		frame.add(addressText);
		frame.add(label4);
		frame.add(portText);
		frame.add(label5);
		frame.add(userLogin);
		frame.add(accept);
		frame.add(cancel);
	}
}

/**
 * <p>
 * Klasa implementujaca lisnera dla JTextField w ktorym wporwadzany bedzie
 * tekst, ktory zostanie wyslany do serwera. Klasa zawiera metody ktore
 * sprawdzac beda dlugosc wprowadzonego tekstu i poprawnosc znakow (wykluczenie
 * polskich znakow).
 * </p>
 * 
 * @author Szymon Lankiewicz
 * 
 */
class TextListener implements KeyListener {
	private JTextField field;
	private int size;

	/**
	 * <p>
	 * Konstruktor klasy
	 * </p>
	 * 
	 * @param f
	 *            Pole tekstowe dla ktorego utworzony ma zostac listener.
	 * @param s
	 *            Dlugosc ciagu jaki moze byc maksymalnie wpisany do pola (-1).
	 * @see Reversi#chatEdit
	 * @see Reversi#Login
	 */
	public TextListener(JTextField f, int s) {
		field = f;
		size = s;
	}

	@Override
	public void keyPressed(KeyEvent arg0) {

	}

	@Override
	public void keyReleased(KeyEvent arg0) {
		if (field.getText().length() > 0) {
			StringBuffer buf = new StringBuffer();
			char[] test = new char[size];
			test = field.getText().toCharArray();
			if ((test[field.getText().length() - 1] < 32)
					|| (test[field.getText().length() - 1] > 125)) {
				for (int i = 0; i < field.getText().length() - 1; i++) {
					buf.append(test[i]);
				}
				field.setText(buf.toString());
			}
			char[] key = new char[size];
			if (field.getText().length() >= size) {
				key = field.getText().toCharArray();
				for (int i = 0; i < size - 1; i++) {
					buf.append(key[i]);
				}
				field.setText(buf.toString());
			} else {
			}
		}
	}

	@Override
	public void keyTyped(KeyEvent arg0) {

	}
}

/**
 * <p>
 * Listener przechwytujacy wprowadzenie (akceptacje) wprowadzonych ustawien w
 * oknie konfiguracyjnym. Jego funkcje pobieraja wprowadzone wartosci i
 * uruchamiaja watek nawiazujacy polaczenie z serwerem.
 * </p>
 * 
 * @author Szymon Lankiewicz
 * @see NetThread
 */
class AcceptListener implements ActionListener {
	private byte gameType;

	private JTextField userLogin;
	private JTextField addressText;
	private JTextField portText;

	public AcceptListener(byte type, JTextField login, JTextField address,
			JTextField port) {
		gameType = type;
		userLogin = login;
		addressText = address;
		portText = port;
	}

	public void actionPerformed(ActionEvent arg0) {
		if ((addressText.getText().isEmpty()) || (portText.getText().isEmpty())
				|| (userLogin.getText().isEmpty())) {
			JOptionPane.showMessageDialog(Reversi.okno,
				    "Some field is empty. Pleas type all info",
				    "Empty field",
				    JOptionPane.ERROR_MESSAGE);
		} else {
			Configuration.w.dispose();
			Reversi.Login = userLogin.getText();
			int port = Integer.parseInt(portText.getText());
			if (Reversi.CONTROL == null) {// jesli watek polaczeniowy nie byl
											// jeszcze nigdy tworzony
				Reversi.CONTROL = new NetThread(addressText.getText(), port,
						gameType); // utworz watek
				Reversi.CONTROL.start();
				System.out.println("Wystartowano polaczenie");
			} else { // jezeli watek byl wczesniej tworzony
				if (!Reversi.CONTROL.isAlive()) { // ale juz nie jest aktywny
					Reversi.CONTROL = new NetThread(addressText.getText(), // to
																			// utworz
																			// watek
																			// jeszcze
																			// raz
							port, gameType);
					Reversi.CONTROL.start();
					System.out.println("Wystartowano polaczenie");
				} else { // i jest aktywny
					Reversi.CONTROL.killThread(); // to deaktywuj watek
					try {
						Reversi.CONTROL.join(); // poczekaj az sie zakonczy
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
					Reversi.CONTROL = new NetThread(addressText.getText(),
							port, gameType); // utworz watek z nowymi
												// parametrami
					Reversi.CONTROL.start();
				}
			}
			Reversi.plansza.repaint();
		}

	}
}

/**
 * <p>
 * Klasa Listnera dla przycisku "CANCEL" w oknie konfiguracyjnym.
 * </p>
 * 
 * @author Szymon Lankiewicz
 * 
 */
class CancelListener implements ActionListener {

	public void actionPerformed(ActionEvent arg0) {
		Configuration.w.dispose();
	}
}

/**
 * <p>
 * Klasa tworzaca okno Top Score.
 * </p>
 * 
 * @author Szymon Lankiewicz
 * 
 */
class TopScore extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	/**
	 * Ramka okna Top Score.
	 */
	static JFrame w = new JFrame("Top score");
	private JButton close = new JButton("Close");

	public TopScore() {
		close.addActionListener(new CloseListener());
	}

	/**
	 * Metoda ustawiajaca parametry okna Top Score.
	 */
	public void showWindow() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {

				JComponent newContentPane = new JPanel();
				newContentPane.setOpaque(false);

				w.setContentPane(newContentPane);
				w.setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
				w.setLayout(null);

				w.setSize(190, 250);
				showGUI(w);
				w.setVisible(true);
			}
		});
	}

	/**
	 * <p>
	 * Metoda dodajaca do okna Top Score, informacje o graczach otrzymane z
	 * serwera
	 * </p>
	 * 
	 * @param frame
	 *            okno Top Score
	 */
	public void showGUI(JFrame frame) {
		frame.setLayout(null);
		StringBuffer buf = new StringBuffer("");
		for (int k=0,i = 0; i < 20; i=i+2,k++) {
			JLabel rank = new JLabel();
			JLabel score = new JLabel();
			buf.append("" + (k+1) + ". ");
			if (Reversi.TOP_SCORE[i] != null) {
				System.out.println(Reversi.TOP_SCORE[i]);
				buf.append(Reversi.TOP_SCORE[i]);
			}
			rank.setText(buf.toString());
			rank.setBounds(5, 20*k, 100, 20);
			frame.add(rank);
			buf.delete(0, buf.length());
			if (Reversi.TOP_SCORE[i+1] != null) {
				System.out.println(Reversi.TOP_SCORE[i+1]);
				buf.append(Reversi.TOP_SCORE[i+1]);
			}
			score.setText(buf.toString());
			score.setBounds(120, 20*k, 100, 20);
			frame.add(score);
			buf.delete(0, buf.length());
		}
		close.setBounds(40, 200, 100, 20);
		frame.add(close);
	}

	/**
	 * <p>
	 * Klasa Listnera dla przycisku "CLOSE" w oknie Top Score.
	 * </p>
	 * 
	 * @author Szymon Lankiewicz
	 */
	class CloseListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent arg0) {
			TopScore.w.dispose();
		}

	}
}

/**
 * <p>
 * Klasa obslugujaca rysowanie tla okna i planszy.
 * </p>
 * 
 * @author Szymon Lankiewicz
 */

class GamePanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	// zmienne do przechowywania plikow png dla elementow planszy (pola, pionki
	// itd.)
	private BufferedImage img = null; // to caej gry
	private BufferedImage logo = null; // logo gry
	private BufferedImage field = null; // pola planszy normalne
	private BufferedImage focusedField = null; // pola planszy podswietlone
	private BufferedImage centerField = null;
	private BufferedImage blackPlayer = null; // pionki gracza czarnego
	private BufferedImage whitePlayer = null; // pionki gracza bialego
	private BufferedImage spLogo = null;
	private BufferedImage mpLogo = null;


	/**
	 * <p>
	 * Wielkosc jednego pola planszy w pixelach.
	 * </p>
	 */
	static int fieldSize = 40;

	/**
	 * <p>
	 * Liczba pol planszy w jednym z wymiarow.
	 * </p>
	 */
	static int fieldNumber = 10;

	/**
	 * <p>
	 * Punkt wyrazony w pikselach w ktorym rozpoczyna sie rysowanie planszy.
	 * </p>
	 */
	static int[] fieldStartPosition = { 100, 100 };

	/**
	 * <p>
	 * Pozycja myszki nad plansza.
	 * </p>
	 */
	static int[] mousePosition = new int[2];

	/**
	 * <p>
	 * Zmienna do kontroli co ma byc wyswietlane na planszy (0 - intro, 1 - gra
	 * SP, 2 - gra MP).
	 * </p>
	 */
	private static int scene = 0;

	/**
	 * <p>
	 * Tablica przechowujaca plansze gdzie: 0 - pole puste, 1 - pionek bialy, 2
	 * - pionek czarny.
	 * </p>
	 */
	static int[][] pions = new int[fieldNumber][fieldNumber];

	/**
	 * <p>
	 * Zmienna okreslajaca czyja kolej: true - gracz, false - przeciwnik.
	 * </p>
	 */
	static int turn = 1;

	/**
	 * <p>
	 * Konstruktor klasy - zadania: dodanie listnera ruchu myszy i wczytanie
	 * grafiki.
	 * </p>
	 */
	public GamePanel() {

		addMouseMotionListener(new MyMouseListener());

		try {
			img = ImageIO.read(Reversi.class.getResource("/plansza.png"));
			logo = ImageIO.read(Reversi.class.getResource("/logo.png"));
			field = ImageIO.read(Reversi.class.getResource("/field.png"));
			focusedField = ImageIO.read(Reversi.class.getResource("/focusedField.png"));
			centerField = ImageIO.read(Reversi.class.getResource("/centerField.png"));
			blackPlayer = ImageIO.read(Reversi.class.getResource("/pionBlack.png"));
			whitePlayer = ImageIO.read(Reversi.class.getResource("/pionWhite.png"));
			spLogo = ImageIO.read(Reversi.class.getResource("/singlePlayer.png"));
			mpLogo = ImageIO.read(Reversi.class.getResource("/multiPlayer.png"));


		} catch (IOException e) {
			e.printStackTrace();
			System.out.println("Nie udalo sie wczytac pliku");
		}

	}

	/**
	 * <p>
	 * Klasa wykrywajaca ruchy myszy i przerysowywujaca podswietlnie planszy
	 * </p>
	 * 
	 * @author Szymon Lankiewicz
	 * 
	 */
	class MyMouseListener implements MouseMotionListener {
		int posX;
		int posY;

		public void mouseMoved(MouseEvent e) {
			posX = e.getX();
			posY = e.getY();
			focuseField(posX, posY);
		}

		public void mouseDragged(MouseEvent e) {

		}
	}

	/**
	 * <p>
	 * Funkcja sprawdzajaca polozenie kursora i przerysowywujaca podswietlenie
	 * planszy.
	 * </p>
	 * 
	 * @param x
	 *            <p>
	 *            Pozycja X kursora.
	 *            </p>
	 * @param y
	 *            <p>
	 *            Pozycja Y kursora.
	 *            </p>
	 */
	private void focuseField(int x, int y) {
		if ((x >= fieldStartPosition[0]) && (y >= fieldStartPosition[1])
				&& (x <= (fieldStartPosition[0] + fieldNumber * fieldSize))
				&& (y <= (fieldStartPosition[1] + fieldNumber * fieldSize))) {
			if ((fieldStartPosition[0]
					+ ((x - fieldStartPosition[0]) / fieldSize) * fieldSize != mousePosition[0])
					|| (fieldStartPosition[1]
							+ ((y - fieldStartPosition[1]) / fieldSize)
							* fieldSize != mousePosition[1])) {
				mousePosition[0] = fieldStartPosition[0]
						+ ((x - fieldStartPosition[0]) / fieldSize) * fieldSize;
				mousePosition[1] = fieldStartPosition[1]
						+ ((y - fieldStartPosition[1]) / fieldSize) * fieldSize;
				repaint(100, 100, 400, 400);
			}

		} else {
			mousePosition[0] = -1;
			mousePosition[1] = -1;
			repaint(100, 100, 400, 400);
		}
	}

	/**
	 * <p>
	 * Funkcja pobierajaca z serwera i aktualizujaca plansze.
	 * </p>
	 * 
	 * @param net
	 *            Lacze z serwerm.
	 */
	public void updatePions(ReversiNetwork net) {
		pions = net.getMatrixFromServer();
		if (Reversi.CONTROL.gameType == 0) {
			net.cleanInput(923);
			turn = 1;
		} else if (Reversi.CONTROL.gameType == 1) {
			turn = net.getTurn();
			System.out.println("KOLEJ: " + turn);
		}
		repaint(100, 100, 400, 400);
	}
	
	public static void clearPions() {
		for (int i=0;i<fieldNumber;i++) {
			for (int j=0;j<fieldNumber;j++) {
				pions[i][j]=0;
			}
		}
	}

	public static void changeScene(int numberOfScene) {
		scene = numberOfScene;
	}

	@Override
	protected void paintComponent(Graphics g) {

		// super.paintComponent(g);
		g.drawImage(img, 0, 0, this);
		if (scene == 0) {
			g.drawImage(logo, 150, 300, this);
		} else if ((scene == 1) || (scene == 2)) {
			for (int i = 0; i < fieldNumber; i++) {
				for (int j = 0; j < fieldNumber; j++) {
					if (((mousePosition[0] > 0) && (mousePosition[1] > 0))
							&& ((i == ((mousePosition[0] - fieldStartPosition[0]) / fieldSize))
									|| (j == ((mousePosition[1] - fieldStartPosition[1]) / fieldSize))
									|| (i
											- ((mousePosition[0] - fieldStartPosition[0]) / fieldSize) == j
											- ((mousePosition[1] - fieldStartPosition[1]) / fieldSize)) || (-1
									* (i - ((mousePosition[0] - fieldStartPosition[0]) / fieldSize)) == j
									- ((mousePosition[1] - fieldStartPosition[1]) / fieldSize)))
							&& ((i != ((mousePosition[0] - fieldStartPosition[0]) / fieldSize)) || (j != ((mousePosition[1] - fieldStartPosition[1]) / fieldSize)))) {
						g.drawImage(focusedField, (i * fieldSize)
								+ fieldStartPosition[0], (j * fieldSize)
								+ fieldStartPosition[1], this);
					} else if ((i != ((mousePosition[0] - fieldStartPosition[0]) / fieldSize))
							|| (j != ((mousePosition[1] - fieldStartPosition[1]) / fieldSize))) {
						g.drawImage(field, (i * fieldSize)
								+ fieldStartPosition[0], (j * fieldSize)
								+ fieldStartPosition[1], this);
					}
				}
			}
			if ((mousePosition[0] > 0) && (mousePosition[1] > 0))
				g.drawImage(centerField, mousePosition[0], mousePosition[1],
						this);
			for (int i = 0; i < fieldNumber; i++) {
				for (int j = 0; j < fieldNumber; j++) {
					if (pions[i][j] == 1) {
						g.drawImage(blackPlayer, fieldStartPosition[0] + i
								* fieldSize, fieldStartPosition[1] + j
								* fieldSize, this);

					}
					else if (pions[i][j] == 2) {
						g.drawImage(whitePlayer, fieldStartPosition[0] + i
								* fieldSize, fieldStartPosition[1] + j
								* fieldSize, this);

					}

				}
			}
			if (scene == 1) {
				g.drawImage(spLogo, 230, 520, this);
			} else if (scene == 2) {
				g.drawImage(mpLogo, 230, 520, this);
			}
		} else {
			System.out.println("Nie ma takiej sceny");
		}
	}

}

/**
 * <p>
 * Klasa Listenra przejmujaca ruchy gracza na planszy. Funkcja klasy pobiera
 * wspolrzedne pola kliknietego przez gracza i wysyla je do serwera.
 * </p>
 * 
 * @author Szymon Lankiewicz
 * 
 */
class MyPlayListener implements MouseListener {

	@Override
	public void mouseClicked(MouseEvent arg0) {

		int x = (GamePanel.mousePosition[0] - GamePanel.fieldStartPosition[0])
				/ GamePanel.fieldSize;
		int y = (GamePanel.mousePosition[1] - GamePanel.fieldStartPosition[1])
				/ GamePanel.fieldSize;
		if ((x >= 0) && (y >= 0)) {

			if (GamePanel.turn == 1) {
				System.out.println("X: " + x + " Y: " + y);
				try {
					Reversi.CONTROL.network.sendMoveToServer(x, y);
				} catch (IOException e) {
					Reversi.CONTROL.network.disconnectFromServer();
				}
				GamePanel.turn = 0;
			}
		}
	}

	@Override
	public void mouseEntered(MouseEvent arg0) {

	}

	@Override
	public void mouseExited(MouseEvent arg0) {

	}

	@Override
	public void mousePressed(MouseEvent arg0) {

	}

	@Override
	public void mouseReleased(MouseEvent arg0) {

	}

}

/**
 * <p>
 * Klasa watku. Dla gracza moze istniec tylko jeden watek kontrolujacy
 * komunikacje z serwerem. Watek odbiera komunikaty z serwera i podejmuje
 * reakcje na nie.
 * </p>
 * 
 * @author Szymon Lankiewicz
 * @see Reversi#CONTROL
 * @see AcceptListener
 */
class NetThread extends Thread {

	String serverAddress;
	int serverPort;
	GamePanel panel;
	JButton sendBut;
	JTextField sendText;
	byte gameType;
	private static byte HEADER_GAME_START = 10;
	ReversiNetwork network;

	/**
	 * <p>
	 * Konstruktor watku pobierajacy wartosci zmiennych wprowadzoncy w oknie
	 * konfiguracyjnym i zapisujacy je w zmiennych statycznych klasy.
	 * </p>
	 * 
	 * @param address
	 *            Adres serwera
	 * @param port
	 *            Port na ktorym dziala serwer
	 * @param type
	 *            Typ gry: 0 - SinglePlayer, 1 - MultiPlayer
	 */
	public NetThread(String address, int port, byte type) {
		serverAddress = address;
		serverPort = port;
		gameType = type;
	}

	/**
	 * <p>
	 * Funkcja wymuszajaca zakonczenie sie watku. Rozlaczajac socketa mozliwe
	 * jest wyskoczenie z nieskonczonej petli i zakonczenie sie poprawne watku.
	 * </p>
	 */
	public void killThread() {
		network.disconnectFromServer();
	}

	@Override
	public void run() {
		MyPlayListener playL = new MyPlayListener();
		try {
			network = new ReversiNetwork(serverAddress, serverPort);
			if (network.connectToServerSimple() != 1) {
				JOptionPane.showMessageDialog(panel,
					    "The supplied address is invalid\n" +
					    "Pleas correct it and try again.",
					    "Error",
					    JOptionPane.ERROR_MESSAGE);
			} else {

				Reversi.plansza.addMouseListener(playL);
				byte[] data = new byte[1024];
				for (int i = 0; i < 1024; i++) {
					data[i] = 0;
				}
				data[0] = HEADER_GAME_START;
				data[1] = gameType;

				network.sendData(data);
				
				if (gameType==0) {
					GamePanel.changeScene(1);
				} else if (gameType==1) {
					GamePanel.changeScene(2);
				}
				byte inst = -1;
				while (inst!=0) {

					inst = network.getInstruction();
					//System.out.println(inst);

					if (inst < 0) {
						network.disconnectFromServer();
						break;
					} else if (inst == 10) {
						network.cleanInput(1023);
						System.out.println("Wszystko ok zaczynamy gre");
					} else if (inst == 1) {
						System.out.println("Update pions");
						Reversi.plansza.updatePions(network);
						int czarne = 0;
						int biale = 0;
						for (int i = 0; i < 10; i++) {
							for (int j = 0; j < 10; j++) {
								if (GamePanel.pions[i][j] == 1) {
									czarne++;
								} else if (GamePanel.pions[i][j] == 2) {
									biale++;
								}
							}
						}
						Reversi.chatWindow.setText("Czarne: " + czarne
								+ " Biale: " + biale);
					} else if (inst == 2) {
						inst = network.getInstruction();
						network.cleanInput(1022);
						if (inst == 0) {
							System.out.println("Przegrales");
							network.sendLogin(Reversi.Login);
							GamePanel.clearPions();
							GamePanel.changeScene(0);
							GamePanel.turn=0;
							Reversi.plansza.repaint();
							JOptionPane.showMessageDialog(panel,
								    "You lose.",
								    "Game Over",
								    JOptionPane.INFORMATION_MESSAGE);
						} else if (inst == 1) {
							System.out.println("Wygrales");
							network.sendLogin(Reversi.Login);
							GamePanel.clearPions();
							GamePanel.changeScene(0);
							GamePanel.turn=0;
							Reversi.plansza.repaint();
							JOptionPane.showMessageDialog(panel,
								    "You win.",
								    "Game Over",
								    JOptionPane.INFORMATION_MESSAGE);
						}
					} else if (inst == 3) {
						System.out.println("Pobieranie top scora");
						Reversi.TOP_SCORE = network.getTopScore();
					} else if (inst == 5) {
						StringBuffer temp = new StringBuffer(Reversi.chatWindow
								.getText());
						temp.append(network.getMessage());
						Reversi.chatWindow.setText(temp.toString());
					} else if (inst == 9) {
						network.disconnectFromServer();
						inst = network.getInstruction();
					} else {
						JOptionPane.showMessageDialog(panel,
							    "Connection Error. Please try again.",
							    "Connection Error",
							    JOptionPane.ERROR_MESSAGE);
						GamePanel.clearPions();
						GamePanel.changeScene(0);
						Reversi.plansza.repaint();
						break;
					}
				}
			}
		} catch (IOException e) {
			network.disconnectFromServer();
			GamePanel.changeScene(0);
		} finally {
			System.out.println("ZABIJAM WATEK");
			Reversi.plansza.removeMouseListener(playL);
		}
	}
}
